@@ -16,20 +16,18 @@ module Codec.Encryption.OpenPGP.Internal (
  , sigPKA
  , sigHA
  , sigCT
- , truncatingVerify
+ , curveoidBSToCurve
+ , curveToCurveoidBS
+ , point2BS
 ) where
 
-import Crypto.Hash (hashWith)
-import qualified Crypto.Hash.IO as CHI
-import Crypto.Number.Basic (numBits)
-import Crypto.Number.ModArithmetic (expFast, inverse)
-import Crypto.Number.Serialize (os2ip)
+import Crypto.Number.Serialize (i2osp, os2ip)
 import qualified Crypto.PubKey.DSA as DSA
+import qualified Crypto.PubKey.ECC.ECDSA as ECDSA
+import qualified Crypto.PubKey.ECC.Types as ECCT
 import qualified Crypto.PubKey.RSA as RSA
 
 import Data.Bits (testBit)
-import Data.ByteArray (ByteArrayAccess)
-import qualified Data.ByteArray as BA
 import qualified Data.ByteString as B
 import Data.ByteString.Lazy (ByteString)
 import qualified Data.ByteString.Lazy as BL
@@ -74,6 +72,8 @@ pubkeyToMPIs (DSAPubKey (DSA_PublicKey k)) = [
   where pkParams f = MPI . f . DSA.public_params $ k
 
 pubkeyToMPIs (ElGamalPubKey p g y) = [MPI p, MPI g, MPI y]
+pubkeyToMPIs (ECDHPubKey ((ECDSA_PublicKey (ECDSA.PublicKey _ q))) _ _) = [MPI (os2ip (point2BS q))]
+pubkeyToMPIs (ECDSAPubKey ((ECDSA_PublicKey (ECDSA.PublicKey _ q)))) = [MPI (os2ip (point2BS q))]
 
 multiplicativeInverse :: Integral a => a -> a -> a
 multiplicativeInverse _ 1 = 1
@@ -100,18 +100,22 @@ sigCT (SigV3 _ ct _ _ _ _ _) = Just ct
 sigCT (SigV4 _ _ _ hsubs _ _ _) = fmap (\(SigSubPacket _ (SigCreationTime i)) -> i) (find isSigCreationTime hsubs)
 sigCT _ = Nothing
 
-truncatingVerify :: (ByteArrayAccess msg, CHI.HashAlgorithm hash) => hash -> DSA.PublicKey -> DSA.Signature -> msg -> Bool
-truncatingVerify hashAlg pk (DSA.Signature r s) m
-    -- Reject the signature if either 0 < r < q or 0 < s < q is not satisfied.
-    | r <= 0 || r >= q || s <= 0 || s >= q = False
-    | otherwise                            = v == r
-    where (DSA.Params p g q) = DSA.public_params pk
-          y       = DSA.public_y pk
-          hm      = os2ip . dsaTruncate . BA.convert $ hashWith hashAlg m
+curveoidBSToCurve :: B.ByteString -> Either String ECCT.Curve
+curveoidBSToCurve oidbs
+    | B.pack [0x2A,0x86,0x48,0xCE,0x3D,0x03,0x01,0x07] == oidbs = Right $ ECCT.getCurveByName ECCT.SEC_p256r1
+    | B.pack [0x2B,0x81,0x04,0x00,0x22] == oidbs = Right $ ECCT.getCurveByName ECCT.SEC_p384r1
+    | B.pack [0x2B,0x81,0x04,0x00,0x23] == oidbs = Right $ ECCT.getCurveByName ECCT.SEC_p521r1
+    | otherwise = Left "unknown curve OID"
 
-          w       = fromJust $ inverse s q
-          u1      = (hm*w) `mod` q
-          u2      = (r*w) `mod` q
-          v       = (expFast g u1 p * expFast y u2 p) `mod` p `mod` q
-          dsaTruncate bs = let lbs = BL.fromStrict bs in if countBits lbs > fromIntegral dsaQLen then B.take (dsaQLen `div` 8) bs else bs -- FIXME: uneven bits
-          dsaQLen = numBits q
+-- [0x2B 0x06 0x01 0x04 0x01 0xDA 0x47 0x0F 0x01] -- ed25519
+
+curveToCurveoidBS :: ECCT.Curve -> Either String B.ByteString
+curveToCurveoidBS curve
+    | curve == ECCT.getCurveByName ECCT.SEC_p256r1 = Right $ B.pack [0x2A,0x86,0x48,0xCE,0x3D,0x03,0x01,0x07]
+    | curve == ECCT.getCurveByName ECCT.SEC_p384r1 = Right $ B.pack [0x2B,0x81,0x04,0x00,0x22]
+    | curve == ECCT.getCurveByName ECCT.SEC_p521r1 = Right $ B.pack [0x2B,0x81,0x04,0x00,0x23]
+    | otherwise = Left "unknown curve"
+
+point2BS :: ECCT.PublicPoint -> B.ByteString
+point2BS (ECCT.Point x y) = B.concat [B.singleton 0x04, i2osp x, i2osp y] -- FIXME: check for length equality?
+point2BS (ECCT.PointO) = error "FIXME: point at infinity"
