@@ -23,6 +23,9 @@ import Crypto.Number.Basic (numBits)
 import Crypto.Number.Serialize (i2osp, os2ip)
 import qualified Crypto.PubKey.RSA as R
 import qualified Crypto.PubKey.DSA as D
+import qualified Crypto.PubKey.ECC.Types as ECCT
+import qualified Crypto.PubKey.ECC.ECDSA as ECDSA
+import Data.Bifunctor (bimap)
 import Data.Bits ((.&.), (.|.), shiftL, shiftR, testBit)
 import Data.ByteString.Lazy (ByteString)
 import qualified Data.ByteString as B
@@ -45,7 +48,7 @@ import Data.Word (Word8, Word32)
 import Data.Maybe (fromMaybe)
 import Network.URI (nullURI, parseURI, uriToString)
 
-import Codec.Encryption.OpenPGP.Internal (pubkeyToMPIs, multiplicativeInverse)
+import Codec.Encryption.OpenPGP.Internal (curveoidBSToCurve, curveToCurveoidBS, pubkeyToMPIs, multiplicativeInverse)
 import Codec.Encryption.OpenPGP.Types
 
 instance Binary SigSubPacket where
@@ -609,8 +612,12 @@ getPkt = do
                           udata <- getByteString (fromIntegral len)
                           return . UserIdPkt . decodeUtf8With lenientDecode $ udata
             | t == 14 = do
-                          pkp <- getPKPayload
-                          return $ PublicSubkeyPkt pkp
+                          bs <- getLazyByteString len
+                          let ps = flip runGetOrFail bs $ do pkp <- getPKPayload
+                                                             return $ PublicSubkeyPkt pkp
+                          case ps of
+                              Left (_, _, err) -> fail ("public subkey " ++ err)
+                              Right (_, _, key) -> return key
             | t == 17 = do
                         bs <- getLazyByteString len
                         case runGetOrFail (many getUserAttrSubPacket) bs of
@@ -783,10 +790,36 @@ getPubkey ForbiddenElgamal = do MPI p <- get
                                 MPI g <- get
                                 MPI y <- get
                                 return $ ElGamalPubKey p g y
+getPubkey ECDSA = do
+    curvelength <- getWord8 -- FIXME: test for 0 or 0xFF as they are reserved
+    curveoid <- getByteString (fromIntegral curvelength)
+    MPI mpi <- getMPI -- FIXME: check length against curve type?
+    case curveoidBSToCurve curveoid >>= \c -> mpi2point c mpi of
+        Left e -> fail e
+        Right (curve, point) -> return . ECDSAPubKey . ECDSA_PublicKey . ECDSA.PublicKey curve $ point
+    where
+        mpi2point c cpi = fmap (\y -> (c,y)) (bs2Point (i2osp cpi))
+getPubkey ECDH = do
+    ECDSAPubKey ed <- getPubkey ECDSA
+    kdflen <- getWord8 -- FIXME: should be 3, test for 0 or 0xFF as they are reserved
+    one <- getWord8 -- FIXME: should be 1
+    kdfHA <- get
+    kdfSA <- get
+    return $ ECDHPubKey ed kdfHA kdfSA
+
 getPubkey _ = UnknownPKey <$> getRemainingLazyByteString
 
+bs2Point :: B.ByteString -> Either String ECDSA.PublicPoint
+bs2Point bs = let xy = B.drop 1 bs in
+                let l = B.length xy in
+                  if B.head bs == 0x04
+                    then return (uncurry ECCT.Point (bimap os2ip os2ip (B.splitAt (div l 2) xy)))
+                    else fail $ "unknown type of point: " ++ show (B.unpack bs)
+
 putPubkey :: PKey -> Put
 putPubkey (UnknownPKey bs) = putLazyByteString bs
+putPubkey p@(ECDSAPubKey (ECDSA_PublicKey (ECDSA.PublicKey curve _))) = let Right curveoidbs = curveToCurveoidBS curve in putWord8 (fromIntegral (B.length curveoidbs)) >> putByteString curveoidbs >> mapM_ put (pubkeyToMPIs p) -- FIXME: do not output length 0 or 0xff
+putPubkey p@(ECDHPubKey (ECDSA_PublicKey (ECDSA.PublicKey curve _)) kha ksa) = let Right curveoidbs = curveToCurveoidBS curve in putWord8 (fromIntegral (B.length curveoidbs)) >> putByteString curveoidbs >> mapM_ put (pubkeyToMPIs p) >> putWord8 0x03 >> putWord8 0x01 >> put kha >> put ksa -- FIXME: do not output length 0 or 0xff
 putPubkey p = mapM_ put (pubkeyToMPIs p)
 
 getSecretKey :: PKPayload -> Get SKey
@@ -806,6 +839,14 @@ getSecretKey pkp
     | _pkalgo pkp `elem` [ElgamalEncryptOnly,ForbiddenElgamal] =
         do MPI x <- get
            return $ ElGamalPrivateKey x
+    | _pkalgo pkp == ECDSA =
+        do MPI pn <- get
+           let pubcurve = (\(ECDSAPubKey (ECDSA_PublicKey p)) -> ECDSA.public_curve p) (pkp^.pubkey)
+           return $ ECDSAPrivateKey (ECDSA_PrivateKey (ECDSA.PrivateKey pubcurve pn))
+    | _pkalgo pkp == ECDH = -- FIXME: deduplicate this and above
+        do MPI pn <- get
+           let pubcurve = (\(ECDSAPubKey (ECDSA_PublicKey p)) -> ECDSA.public_curve p) (pkp^.pubkey)
+           return $ ECDHPrivateKey (ECDSA_PrivateKey (ECDSA.PrivateKey pubcurve pn))
 
 putSKey :: SKey -> Put
 putSKey (RSAPrivateKey (RSA_PrivateKey (R.PrivateKey _ d p q _ _ _))) = put (MPI d) >> put (MPI p) >> put (MPI q) >> put (MPI u)
