@@ -46,6 +46,7 @@ import Data.Monoid (Monoid, mempty)
 #endif
 import Data.Monoid ((<>))
 import Data.Ord (comparing)
+import Data.Semigroup (Semigroup)
 import Data.Set (Set)
 import qualified Data.Set as Set
 import Data.Text (Text)
@@ -868,7 +869,7 @@ instance Pretty KeyVersion where
 $(ATH.deriveJSON ATH.defaultOptions ''KeyVersion)
 
 newtype IV = IV {unIV :: B.ByteString}
-    deriving (ByteArrayAccess, Data, Eq, Generic, Hashable, Monoid, Show, Typeable)
+    deriving (ByteArrayAccess, Data, Eq, Generic, Hashable, Semigroup, Monoid, Show, Typeable)
 
 instance Newtype IV B.ByteString where
     pack = IV
